@@ -28,17 +28,27 @@ bolded_colours = {BLACK: (0x80, 0x80, 0x80),
                   CYAN: (0x00, 0xFF, 0xFF),
                   WHITE: (0xFF, 0xFF, 0xFF)}
 
+_fg_cache = {}
 def fg_code(code, bold):
     """Wrapper to convert a VT100 colour code to a HexFGCode."""
+    if (code, bold) in _fg_cache:
+        return _fg_cache[(code, bold)]
     if bold:
         dictionary = bolded_colours
     else:
         dictionary = normal_colours
-    return HexFGCode(*dictionary[code])
+    res = HexFGCode(*dictionary[code])
+    _fg_cache[(code, bold)] = res
+    return res
 
+_bg_cache = {}
 def bg_code(code):
     """Wrapper to convert a VT100 colour code to a HexBGCode."""
-    return HexBGCode(*normal_colours[code])
+    if code in _bg_cache:
+        return _bg_cache[code]
+    res = HexBGCode(*normal_colours[code])
+    _bg_cache[code] = res
+    return res
 
 class _HexCode(object):
     """Base class for hex colour codes.
@@ -47,12 +57,15 @@ class _HexCode(object):
     is brightest.
     """
 
-    __slots__ = ['red', 'green', 'blue']
+    ground = None
 
     def __init__(self, red, green, blue):
         self.red = red
         self.green = green
         self.blue = blue
+        #__hash__ gets called in an inner loop, so cache the value.
+        self.as_hex = ''.join(('%x' % num).zfill(2) for num in self.triple)
+        self._hashed = hash(self.ground) ^ hash(self.triple)
 
     def __eq__(self, other):
         return type(self) == type(other) and self.triple == other.triple
@@ -72,7 +85,7 @@ class _HexCode(object):
         return "<%s %s>" % (type(self).__name__, self.tohex())
 
     def __hash__(self):
-        return hash(type(self)) ^ hash(self.triple)
+        return self._hashed
 
     @property
     def triple(self):
@@ -84,13 +97,13 @@ class _HexCode(object):
 
         Suitable for embedding the colours into HTML.
         """
-        return ''.join(('%x' % num).zfill(2) for num in self.triple)
+        return self.as_hex
 
 class HexBGCode(_HexCode):
     """A hex background colour."""
-    pass
+    ground = 'back'
 
 class HexFGCode(_HexCode):
     """A hex foreground colour."""
-    pass
+    ground = 'fore'
 
