@@ -1,5 +1,5 @@
 """Contains the widget that displays the text from the MUD."""
-from mudpyl.metaline import pairwise
+from itertools import izip, chain
 import gtk
 import pango
 
@@ -38,7 +38,7 @@ class OutputView(gtk.TextView):
         """Stop autoscrolling to new data."""
         if not self.paused:
             self.paused = True
-            self.gui.paused_label.set_text("PAUSED")
+            self.gui.paused_label.set_markup("PAUSED")
 
     def unpause(self):
         """Restart autoscrolling to new data.
@@ -47,11 +47,11 @@ class OutputView(gtk.TextView):
         """
         if self.paused:
             self.paused = False
-            self.gui.paused_label.set_text("")
+            self.gui.paused_label.set_markup("")
         #scroll to the end of output
         self.scroll_mark_onscreen(self.end_mark)
 
-    def peek_metaline(self, metaline):
+    def show_metaline(self, metaline):
         """Write a span of text to the window using the colours defined in
         the other channels.
 
@@ -64,17 +64,19 @@ class OutputView(gtk.TextView):
         self.apply_colours(metaline.backs, offset, len(metaline.line))
         if not self.paused:
             self.scroll_mark_onscreen(self.end_mark)
+        else:
+            self.gui.paused_label.set_markup("<span foreground='#FFFFFF' "
+                                                   "background='#000000'>"
+                                               "MORE - PAUSED</span>")
 
     def apply_colours(self, colours, offset, end_offset):
         """Apply a RunLengthList of colours to the buffer, starting at
         offset characters in.
         """
-        #add a dummy item at the end to make sure that even the very last
-        #of the metaline is covered
-        values = colours.as_pruned_index_list()
-        values.append((end_offset, None))
         end_iter = self.buffer.get_iter_at_offset(offset)
-        for (start, colour), (end, _) in pairwise(values):
+        for colour, end in izip(colours.itervalues(),
+                                colours.keys()[1:] + [end_offset]):
+            #TODO: try this with the recreating iter approach
             tag = self.fetch_tag(colour)
             start_iter = end_iter
             end_iter = self.buffer.get_iter_at_offset(end + offset)
@@ -83,7 +85,7 @@ class OutputView(gtk.TextView):
     def fetch_tag(self, colour):
         """Check to see if a colour is in the tag table. If it isn't, add it.
 
-        Return's the tag.
+        Returns the tag.
         """
         #use our own tag table, because GTK's is too slow
         if colour in self._tags:
