@@ -41,36 +41,38 @@ class LineAlterer(object):
         """Change a span's foreground."""
         self._changes.append(('change_fore', start, end, colour))
 
+    def insert_metaline(self, start, metaline):
+        """Insert a coloured metaline."""
+        self._changes.append(("insert_metaline", start, metaline))
+
     def change_back(self, start, end, colour):
         """Change a span's background."""
         self._changes.append(('change_back', start, end, colour))
 
-    def _alter(self, start, adjustment):
+    def _alter(self, start, adj):
         """Change the indices to account for deletions and insertions."""
-        res = deque()
-        for change in self._changes:
+        for ind, change in enumerate(self._changes):
             if change[0] == 'delete':
                 meth, mstart, mend = change
-                res.append((meth, iadjust(mstart, start, adjustment), 
-                            iadjust(mend, start, adjustment)))
-            elif change[0] == 'insert':
+                self._changes[ind] = (meth, iadjust(mstart, start, adj), 
+                                      iadjust(mend, start, adj))
+            elif change[0] in ('insert', 'insert_metaline'):
                 meth, mstart, text = change
-                res.append((meth, iadjust(mstart, start, adjustment), text))
+                self._changes[ind] = (meth, iadjust(mstart, start, adj), text)
             elif change[0] in ('change_fore', 'change_back'):
                 meth, mstart, mend, colour = change
-                res.append((meth, iadjust(mstart, start, adjustment), 
-                            iadjust(mend, start, adjustment), colour))
-        self._changes = res
+                self._changes[ind] = (meth, iadjust(mstart, start, adj), 
+                                      iadjust(mend, start, adj), colour)
 
     def apply(self, metaline):
         """Apply our changes to a metaline.
 
         This LineAlterer is no good after doing this, so copy it if it needs
-        to be reused.
+        to be reused. The metaline passed in, however, is left pristine.
         """
-        metaline = metaline.copy()
-        while self._changes:
-            change = self._changes.popleft()
+        if self._changes:
+            metaline = metaline.copy()
+        for change in self._changes:
             meth = change[0]
             args = change[1:]
             if meth == 'delete':
@@ -80,6 +82,9 @@ class LineAlterer(object):
             elif meth == 'insert':
                 start, text = args
                 self._alter(start, len(text))
+            elif meth == "insert_metaline":
+                start, ins_metaline = args
+                self._alter(start, len(ins_metaline.line))
             getattr(metaline, meth)(*args)
         return metaline
 
@@ -104,8 +109,8 @@ class TriggerMatchingRealm(BaseMatchingRealm):
     .parent, which is the Realm up one level from this one.
     """
 
-    def __init__(self, metaline, root, parent):
-        BaseMatchingRealm.__init__(self, root, parent)
+    def __init__(self, metaline, root, parent, send_line_to_mud):
+        BaseMatchingRealm.__init__(self, root, parent, send_line_to_mud)
         self.metaline = metaline
         self.alterer = LineAlterer()
         self.display_line = True
@@ -116,13 +121,12 @@ class TriggerMatchingRealm(BaseMatchingRealm):
         metaline = self.alterer.apply(self.metaline)
         if self.display_line:
             self.parent.write(metaline)
-        for noteline, sls in self._writing_after:
-            self.parent.write(noteline, sls)
+        self._write_after()
 
     def send(self, line, echo = False):
         """Send a line to the MUD."""
         #need to spin a new realm out here to make sure that the writings from
         #the alias go after ours.
-        realm = AliasMatchingRealm(line, echo, parent = self, 
-                                   root = self.root)
+        realm = AliasMatchingRealm(line, echo, parent = self, root = self.root,
+                                   send_line_to_mud = self.send_line_to_mud)
         realm.process()
